const storage = {};
/**
 * hotkey class instance
 * @param alternateUp callback
 */
class HotkeyInstance {
    constructor(alternateUp) {
        this.up =
            alternateUp !== undefined && alternateUp !== undefined
                ? alternateUp
                : null; // Function
        this.down = null; // Function
        this.pressed = false; // Boolean - only let us press once
    }
    up;
    down;
    pressed;
}
const getCommandString = ({ altKey, ctrlKey, shiftKey, key, }) => {
    let value = '';
    const characterMap = {
        '~': '`',
        '!': '1',
        '@': '2',
        '#': '3',
        $: '4',
        '%': '5',
        '^': '6',
        '&': '7',
        '*': '8',
        '(': '9',
        ')': '0',
        _: '-',
        '+': '=',
        '{': '[',
        '}': ']',
        '|': '\\',
        ':': ';',
        '<': ',',
        '>': '.',
        '?': '/',
    };
    if (key !== undefined) {
        const keyInCharacterMap = characterMap[key];
        if (keyInCharacterMap !== undefined) {
            key = keyInCharacterMap;
        }
    }
    if (altKey !== undefined && altKey && key !== 'alt') {
        value += 'alt+';
    }
    if (ctrlKey !== undefined && ctrlKey && key !== 'control') {
        value += 'ctrl+';
    }
    if (shiftKey !== undefined && shiftKey && key !== 'shift') {
        value += 'shift+';
    }
    if (key !== undefined)
        key = key?.toLowerCase(); // Force lowercase
    // Handle space and regular keysg
    if (key !== undefined) {
        value += key === ' ' ? 'space' : key;
    }
    return value;
};
const formatCommandString = (command) => {
    // Collect Modifiers
    const commands = {};
    command.split('+').forEach((item) => {
        if (item === 'alt')
            commands.altKey = true;
        else if (item === 'ctrl') {
            commands.ctrlKey = true;
        }
        else if (item === 'control') {
            commands.ctrlKey = true;
        }
        else if (item === 'shift') {
            commands.shiftKey = true;
        }
        else
            commands.key = item; // Store regular keys but add something so we know it uses modifiers
    });
    // reformat storage name
    return getCommandString(commands);
};
// bindEvents
const bindEvents = () => {
    // Key down event listener
    document.addEventListener('keydown', (event) => {
        const commandString = getCommandString(event);
        const hotkey = storage[commandString];
        if (hotkey !== undefined && !hotkey.pressed) {
            if (hotkey.down !== null)
                hotkey.down(event);
            hotkey.pressed = true;
        }
    });
    // Key up event listener
    document.addEventListener('keyup', (event) => {
        const hotkey = storage[getCommandString(event)];
        if (hotkey !== undefined) {
            if (hotkey.up !== null)
                hotkey.up(event);
            hotkey.pressed = false;
        }
    });
};
// Initialization boolean for doing things only once
let initialized = false;
// Save last key so we can find it for adding up and down methods
let lastKey = null;
/**
 * Set one hotkey
 * @param command string hotkey command
 * @param alternateUp  callback
 */
const hotkey = (command, alternateUp) => {
    // Initialize only once
    if (!initialized) {
        bindEvents();
        initialized = true;
    }
    // Command is combined: ctlr+z
    if (command.length > 1 && command.includes('+')) {
        lastKey = formatCommandString(command);
        // Command is single modifier: shift, ctrl, space enter, backspace, ect
        // Or its a single letter a, b, c, d
    }
    else {
        lastKey = command;
    }
    // Create blank new hotkey and use callback if provided here
    if (storage[lastKey] === undefined) {
        storage[lastKey] = new HotkeyInstance(alternateUp !== undefined ? alternateUp : null);
    }
    // Allow changing methods
    return { up, down };
};
/**
 * Set callback for down
 * @param callback () => void
 */
const down = (callback) => {
    // Store "down" callback with last command
    if (callback !== undefined && lastKey !== null) {
        storage[lastKey].down = callback;
    }
    // Allow chaining methods
    return { up };
};
/**
 * Set callback for up
 * @param callback () => void
 */
const up = (callback) => {
    // Store "up" callback with last command
    if (callback !== undefined && lastKey !== null) {
        storage[lastKey].up = callback;
    }
    // Allow changing methods
    return { down };
};
// Chain removal function
hotkey.remove = (command) => {
    if (storage[formatCommandString(command)] !== undefined) {
        delete storage[formatCommandString(command)];
    }
};

export { HotkeyInstance, bindEvents, formatCommandString, getCommandString, hotkey, storage };
